////////////////////////////////////////////////////////////////////////////
//  decframe.cpp
//
//  This file is part of the WinNT Pipes example demonstrating how to 
//  use anonymous pipes to communicate between a parent and child 
//  process through the child's STDIN/STDOUT. 
//
//  George Cross, Borland C++ Developer Support March 97
//
//  Built with Borland C++ 5.01
////////////////////////////////////////////////////////////////////////////

#include <owl\pch.h>
#pragma hdrstop
#include <owl\edit.h>
#include <owl\opensave.h>
#include <stdio.h>
#include <string.h>
#include "decframe.h"

////////////////////////////////////////////////////////////////////////////
// TPipes32Frame class for the main window of this application
////////////////////////////////////////////////////////////////////////////
DEFINE_RESPONSE_TABLE1 (TPipes32Frame, TDecoratedFrame)
  EV_COMMAND (CM_RUNTHEPIPE, CmRunThePipe),
END_RESPONSE_TABLE;

//
//Class constructor definition
TPipes32Frame::TPipes32Frame ()
              : TDecoratedFrame (NULL,                  //No parent yet
                                 NULL,                  //Caption
                                 new TEdit (NULL,       //TEdit client
                                            IDC_EDIT1,  //Arbitrary ID
                                            NULL,       //Caption
                                            0,          //No dimensions
                                            0,          //will make it cover
                                            0,          //entire client area
                                            0,
                                            0,          //No text limit
                                            TRUE))      //Multiline
{
  FileData.Flags = OFN_FILEMUSTEXIST |         //Set values for the
                   OFN_HIDEREADONLY  ;         //OpenFile dialog
  FileData.SetFilter ("Source files |*.cpp|Text files |*.txt|");
  *FileData.FileName = 0;
};                                             //End TPipes32Frame constructor


//
//CmFileNew response function
void TPipes32Frame::CmRunThePipe () {

  //
  //Find the ASCII file to pass to STDIN of child process 
  if (TFileOpenDialog (this, FileData).Execute () != IDOK) 
    return;
  HANDLE hFHandle;
  hFHandle = CreateFile(FileData.FileName,
                            GENERIC_READ,
                            FILE_SHARE_READ,
                            NULL,
                            OPEN_EXISTING,
                            NULL,
                            NULL);
  if (hFHandle == INVALID_HANDLE_VALUE){
    dynamic_cast<TEdit*>(GetClientWindow())->Insert ("Error opening file !.\n");
    return;
  }

  bool bReturnValue;                       //Success/Fail flag
  char szbuffer[1024];
  unsigned long nbytesread;
  memset(szbuffer,0,sizeof(szbuffer));
  bReturnValue = ReadFile(hFHandle,
                          szbuffer,
                          sizeof(szbuffer)-1,
                          &nbytesread,
                          NULL);
  
  if (!bReturnValue){ 
    dynamic_cast<TEdit*>(GetClientWindow())->Insert ("Error reading file.\n");
    return;
  }
  
  dynamic_cast<TEdit*>(GetClientWindow())->Insert (szbuffer);
  dynamic_cast<TEdit*>(GetClientWindow())->Insert ("\r\n\r\nSending above "
                                                   "data to pipe. . .\r\n\r\n");
 
 
  SECURITY_ATTRIBUTES saAttr;              //Security attributes of handles
  PROCESS_INFORMATION piProcInfo;          //
  STARTUPINFO         siStartInfo;         //

  memset(&saAttr,0,sizeof(SECURITY_ATTRIBUTES));
  saAttr.nLength = sizeof (SECURITY_ATTRIBUTES);//Structure size
  saAttr.bInheritHandle = TRUE;                 //Permit handles to be inherited
  saAttr.lpSecurityDescriptor = NULL;           //No descriptor necessary

  HANDLE hParentRd, hChildStdOut;               //Handles of the pipe
  bReturnValue = CreatePipe (&hParentRd,        //Handle for reading
                             &hChildStdOut,     //Handle for writing
                             &saAttr,           //Security attributes
                             0);                //Default buffer size

  if (!bReturnValue){ 
    dynamic_cast<TEdit*>(GetClientWindow())->Insert ("Error creating output "
                                                     "pipe.\n");
    return;
  }

  HANDLE hChildStdIn, hParentWr;             
  bReturnValue = CreatePipe (&hChildStdIn, 
                             &hParentWr, 
                             &saAttr, 
                             0);   

  if (!bReturnValue){ 
    dynamic_cast<TEdit*>(GetClientWindow())->Insert ("Error creating "
                                                     "input pipe.\n");
    CloseHandle(hParentRd);
    CloseHandle(hChildStdOut);
    return;
  }

  //Beware that strlen does not include the '\0' in its 
  //calculation.  So when calling WriteFile to put a string to a pipe, 
  //you might want to add one to the number of bytes (+1) if using strlen to 
  //calculate this value.

  unsigned long bwritten;
  bReturnValue = WriteFile(hParentWr,
                           szbuffer,
                           strlen(szbuffer),
                           &bwritten,
                           NULL);
  
  if (!bReturnValue){ 
    dynamic_cast<TEdit*>(GetClientWindow())->Insert ("Error writing file "
                                                     "to pipe.\n");
    CloseHandle(hParentRd);
    CloseHandle(hChildStdOut);
    CloseHandle(hChildStdIn);
    CloseHandle(hParentWr);
    return;
  }

  //Put an end of file character in there also.
  szbuffer[0] = 0x1A;
  bReturnValue = WriteFile(hParentWr,
                           szbuffer,
                           1,
                           &bwritten,
                           NULL);
  
  
  if (!bReturnValue){ 
    dynamic_cast<TEdit*>(GetClientWindow())->Insert ("Error writing file "
                                                     "to pipe.\n");
    CloseHandle(hParentRd);
    CloseHandle(hChildStdOut);
    CloseHandle(hChildStdIn);
    CloseHandle(hParentWr);
    return;
  }

  memset(&siStartInfo,0,sizeof(STARTUPINFO));
  siStartInfo.cb = sizeof(STARTUPINFO);
  siStartInfo.dwFlags = STARTF_USESTDHANDLES;
  siStartInfo.hStdInput = hChildStdIn;       //STDIN handle for child 
  siStartInfo.hStdOutput = hChildStdOut;     //STDOUT handle for child


  bReturnValue = CreateProcess (NULL,        //use command line instead
                                "child.exe", //command line
                                NULL,        //process security attributes
                                NULL,        //primary thread security attrs
                                TRUE,        //handles are inherited
                                DETACHED_PROCESS, //creation flags
                                NULL,        //use parent's environment
                                NULL,        //use parent's current directory
                                &siStartInfo,//STARTUPINFO pointer
                                &piProcInfo);//receives PROCESS_INFO

  if (! bReturnValue) {
    dynamic_cast<TEdit*>(GetClientWindow())->Insert ("Error creating "
                                                     "process.\n");
    CloseHandle(hChildStdIn);
    CloseHandle(hParentWr);
    CloseHandle(hParentRd);
    CloseHandle(hChildStdOut);
    return;
  }

  CloseHandle(hChildStdOut);  //Close handles we gave to the child so we  
  CloseHandle(hChildStdIn);   //don't use them

  //
  //Now read the STDOUT of the created process into our edit window.
  //
  do{
    memset(szbuffer,0,sizeof(szbuffer));
    bReturnValue = ReadFile(hParentRd,szbuffer,sizeof(szbuffer)-1,
                            &nbytesread,NULL);
    
    dynamic_cast<TEdit*>(GetClientWindow())->Insert (szbuffer);
  }while(bReturnValue && nbytesread!=0);

  CloseHandle(hParentRd);                 //Close handles
  CloseHandle(hParentWr); 
  CloseHandle(hFHandle);
}                                         //End CmRunThePipe

